/*
	EZ-USB BulkRead / Bulk Write Handler

	2003.1.14 Tatsuyuki Satoh , created
	2003.3. 2 Tatsuyuki Satoh , fixed WindowsXP boot proglem.
	2003.3.12 Tatsuyuki Satoh , use Signal for read finish

	Based by Chamereon-USB Ver1.1 by optimize
*/
#include <windows.h>
#include <stdio.h>
#include <process.h>
#include <winioctl.h>

#include "ezusb.h"

typedef unsigned char u8;
typedef signed char s8;
typedef unsigned short int u16;
typedef signed short int s16;
typedef unsigned int u32;
typedef signed int s32;


// ReadThread
static BYTE *read_ptr;  // read pointer , NULL=terminate thread
static DWORD read_size; // read size from USB
static HANDLE rt_event; // Event Handler for wakeup read_thread
static HANDLE rf_event; // Event Handler for wakeup read_thread
static BULK_TRANSFER_CONTROL bulk_read_control;

// handle for EZ-USB
static HANDLE usb_h  = NULL;
static HANDLE usb_rh = NULL;

static s32 usb_open(s32 n,HANDLE *h){
	s8 dev_name[256];
	sprintf((char *)dev_name,(char *)"\\\\.\\ezusb-%d",n);
	*h = CreateFile((char *)dev_name,
		GENERIC_WRITE,
		FILE_SHARE_WRITE,
		NULL,
		OPEN_EXISTING,
		0,
		NULL);
	if(*h == INVALID_HANDLE_VALUE) {
		return(-1);
	}
	return(0);
}

static s32 usb_close(HANDLE *h){
   CloseHandle(*h);
   return(0);
}

static s32 usb_halt(HANDLE *h){
	unsigned long nbyte;
	BOOLEAN ret = FALSE;
	VENDOR_REQUEST_IN vreq;

	vreq.bRequest = 0xA0;
	vreq.wValue = 0x7F92;
	vreq.wIndex = 0x00;
	vreq.wLength = 0x01;
	vreq.bData = 1;
	vreq.direction = 0x00;
	ret = DeviceIoControl (*h,
							IOCTL_Ezusb_VENDOR_REQUEST,
							&vreq,
							sizeof(VENDOR_REQUEST_IN),
							NULL,
							0,
							&nbyte,
							NULL);
	if(ret==FALSE){
		printf("i8051 halt err.\n");
		return(-1);
	}
	return(0);
}

static s32 usb_run(HANDLE *h){
	unsigned long nbyte;
	BOOLEAN ret = FALSE;
	VENDOR_REQUEST_IN vreq;

	vreq.bRequest = 0xA0;
	vreq.wValue = 0x7F92;
	vreq.wIndex = 0x00;
	vreq.wLength = 0x01;
	vreq.bData = 0;
	vreq.direction = 0x00;
	ret = DeviceIoControl (*h,
							IOCTL_Ezusb_VENDOR_REQUEST,
							&vreq,
							sizeof(VENDOR_REQUEST_IN),
							NULL,
							0,
							&nbyte,
							NULL);
	if(ret==FALSE){
		printf("i8051 run err.\n");
		return(-1);
	}
	return(0);
}

static s32 usb_dwnload(HANDLE *h,u8 *image,s32 len){
	unsigned long nbyte;
	BOOLEAN ret = FALSE;
	
	ret = DeviceIoControl (*h,
							IOCTL_Ezusb_ANCHOR_DOWNLOAD,
							image,
							len,
							NULL,
							0,
							&nbyte,
							NULL);
	if(ret==FALSE){
		printf("usb dwnload err.\n");
		return(-1);
	}
	return(0);
}

static s32 usb_resetpipe(HANDLE *h,ULONG p){
	unsigned long nbyte;
	BOOLEAN ret = FALSE;
	
	ret = DeviceIoControl (*h,
							IOCTL_Ezusb_RESETPIPE,
							&p,
							sizeof(ULONG),
							NULL,
							0,
							&nbyte,
							NULL);
	if(ret==FALSE){
		return(-1);
	}
	return(0);
}

static s32 usb_get_string(HANDLE *h,s32 idx,s8 *s){
	unsigned long nbyte;
	s32 i;
	u8  pvbuf[2+128];
	GET_STRING_DESCRIPTOR_IN sin;
	BOOLEAN ret = FALSE;

	sin.Index=idx;
	sin.LanguageId=27;
	ret = DeviceIoControl (*h,
							IOCTL_Ezusb_GET_STRING_DESCRIPTOR,
							&sin,
							sizeof(GET_STRING_DESCRIPTOR_IN),
							pvbuf,
							sizeof (pvbuf),
							&nbyte,
							NULL);
	if(ret==FALSE){
		return(-1);
	}
	for(i=0;i<pvbuf[0]/2-1;i++){
		*(s++)=(s8)pvbuf[2+i*2];
	}
	*s=0;
	return(0);
}

///////////////////////////////////////////////////////////////////////////////

int ezusb_bulk_write(DWORD pipe,void *pdata,DWORD size)
{
	DWORD block_size;
	BOOLEAN ret = FALSE;
	BULK_TRANSFER_CONTROL bulk_control;

	bulk_control.pipeNum = pipe;
	while(size)
	{
		block_size = size > 0x8000 ? 0x8000 : size;
		ret = DeviceIoControl (usb_h,
							IOCTL_EZUSB_BULK_WRITE,
							&bulk_control,
							sizeof(BULK_TRANSFER_CONTROL),
							pdata,
							block_size,
							&block_size,
							NULL);
		if(ret==FALSE)
			return(-1);

		pdata += block_size;
		size -= block_size;
	}
	return(0);
}

// Close & Shutdown USB handler
void ezusb_close(void)
{
	// Finish ReadThread
	read_ptr = NULL;
	SetEvent(rt_event);

	if(usb_h)
		CloseHandle(usb_h);

	if(usb_rh)
		CloseHandle(usb_rh);

	CloseHandle(rf_event);
	CloseHandle(rt_event);
}

void ezusb_read_thread(void *p)
{
	DWORD block_size;
	int ret;

	// Set ThreadPriority
	SetThreadPriority(GetCurrentThread(), THREAD_PRIORITY_TIME_CRITICAL);

	while(1)
	{
		// wait for power up
		read_ptr = NULL;
		// set finish signale
		SetEvent(rf_event);
		// Wait for next read
		WaitForSingleObject(rt_event,INFINITE);
		ResetEvent(rt_event);

		// Terminate request?
		if(!read_ptr)
			break;

		// Go bulk read form EZ-USB
		while(read_size)
		{
			block_size = read_size > 0x8000 ? 0x8000 : read_size;

			ret = DeviceIoControl (usb_rh,
								IOCTL_EZUSB_BULK_READ,
								&bulk_read_control,
								sizeof(BULK_TRANSFER_CONTROL),
								read_ptr,
								block_size,
								&block_size,
								NULL);
			// error check
			if(ret==FALSE)
			{
				read_size = 0xffffffff;
				break;
			}
			read_ptr  += block_size;
			read_size -= block_size;
		}
	}
}

// start bulk read
void ezusb_bulk_read(DWORD pipe,void *pdata,DWORD size)
{
	// Wait for Current Read
	if( ezusb_bulk_read_left(TRUE) != 0)
		return; /* error */

	// Reset Finish Signal
	ResetEvent(rf_event);

	// Wakeup Read Thread
	read_ptr  = (BYTE *)pdata;
	read_size = size;
	bulk_read_control.pipeNum = pipe;
	SetEvent(rt_event);
}

// check bulk read finish
DWORD ezusb_bulk_read_left(int is_wait_finish)
{
	if(is_wait_finish)
	{
		WaitForSingleObject(rf_event,5000);
	}
	return read_size;
}

int ezusb_init(DWORD n,BYTE *fw,char *str1,char *str2)
{
	char s[128];
	int retry;
	DWORD threadid;
	int after_boot = 0;

	for(retry=0;retry<200;retry++)
	{
		if(usb_open(n,&usb_h))
		{	/* open error */
			/* is first open ? */
			if(retry==0)
				return (-1); /* error! */
			/* check again */
			Sleep(50);
		}
		else
		{
			if(after_boot)
			{
printf("-Reconnect");
				after_boot = 0;
				/* wait 500ms after reconnect */
				usb_close(&usb_h);
				Sleep(500);
				continue;
			}

			/* compare firmware name */
			if(!usb_get_string(&usb_h,1,s) &&
			   !strcmp((const char *)str1,(const char *)s) &&
			   !usb_get_string(&usb_h,2,s) &&
			   !strcmp((const char *)str2,(const char *)s) )
			{	/* match! */

				// Initialize & start ReadThread
				usb_open(n,&usb_rh);
				rt_event = CreateEvent(0,TRUE,FALSE,0);
				rf_event = CreateEvent(0,TRUE,FALSE,0);
				read_ptr = NULL;
				read_size = 0;
				CreateThread(0,0,(LPTHREAD_START_ROUTINE)ezusb_read_thread,0,0,&threadid);
				// Done!
				return(0);
			}
#if 0
			return (-1); /* error! */
#endif
			/* download & reboot */
printf("-Boot");
			if(usb_halt(&usb_h) ||
			  usb_dwnload(&usb_h,fw,CUSB_DWLSIZE) ||
			  usb_run(&usb_h))
			{
				usb_close(&usb_h);
				return (-1); /* error! */
			}
			usb_close(&usb_h);
			after_boot = 1;
			Sleep(1500);
printf("-Wait");
		}
 printf(".");
	}
	/* timeout */
	return(-1);
}



